package com.jeeplus.modules.sg.overheadline.service;

import com.alibaba.fastjson.JSONArray;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.utils.sg.ExcelUtil;
import com.jeeplus.modules.sg.overheadline.entity.*;
import com.jeeplus.modules.sg.overheadline.mapper.OverheadLineMapper;
import com.jeeplus.modules.sg.overheadline.util.BashInfo;
import com.jeeplus.modules.sg.overheadline.util.DetailFeeUtil;
import com.jeeplus.modules.sg.overheadline.util.DoubleUtil;
import com.jeeplus.modules.sg.overheadline.util.SettementUtil;
import com.jeeplus.modules.sg.project.entity.WbsItem;
import com.jeeplus.modules.sg.project.service.ItemService;
import com.jeeplus.modules.sg.substation.util.SubstationInfo;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Service
public class OverheadLineService {
    @Autowired
    private OverheadLineMapper overheadLineMapper;
    @Autowired
    private FeeAdjustService feeAdjustService;
    @Autowired
    private SettlementService settlementService;
    @Autowired
    private ItemService itemService;

    /**
     * 获取wbs节点列表
     * @param id            项目id，必须有
     * @param state         单独调整展示状态 0：不展示 1：展示
     *                      基本上是 除掉设备购置费的本体工程、其他费用、项目类型节点的和
     * @param feeType       节点类型：0.其他费用节点；1.本体节点（建筑、安装、设备购置费）；
     *                      2.项目类型节点（一个项目类型只有一个）；3.项目中剩余费用节点（如：基本预备费）；4.顶级节点（一个）
     * @param projectType   项目类型：1。架空；2.电缆；3.变电站
     * @param totalFee      总计，只要有任何值，则查询节点totalFee不为空
     * @return
     */
    public List<JkxlDetailFee> getDetails(String id,String state,
                                   String feeType,String projectType,String totalFee){
       return overheadLineMapper.getDetails(id,state,feeType,projectType,totalFee);
    }

    /**
     * 获取项目节点列表，默认展示
     * @param id
     * @param feeType
     * @param totalFee
     * @return
     */
    public List<JkxlDetailFee> getDetails(String id,String feeType,String totalFee){
        return overheadLineMapper.getDetails(id,"1",feeType,null,totalFee);
    }

    /**
     * 处理分摊调整
     * @param ftData
     */
    public void shareAdjust(FtData ftData){
        String id = ftData.getId();
        String code = ftData.getCode();   //分摊费用类型
        String type = ftData.getType();   //分摊费用名
        String itemType = ftData.getItemType();   //项目类型
        String state = ftData.getState();  //1：增加 2：减少
        double ftFee = ftData.getFee();   //分摊金额
        if(ftFee==0.00){
            return;
        }
        if(state.equals("2")){
            ftFee = -ftFee;
        }
        List<JkxlDetailFee> list = getDetails(id,"1","1",itemType,"1");
        JkxlDetailFee top = new JkxlDetailFee();
        //处理变电站调整
        if(itemType.equals("3")){
            top.setWbsCode("20000000");
            List<JkxlDetailFee> bdzList = new ArrayList<>();
            for(JkxlDetailFee fee:list){
                if(fee.getWbsCode().equals("21000000")||fee.getWbsCode().equals("23000000")){
                    bdzList.add(fee);
                }
            }
            DetailFeeUtil.getGatherFee(top,bdzList);
            list.add(top);
            DetailFeeUtil.getTree(list);
        }else{
            //处理架空和电缆线路
            DetailFeeUtil.getGatherFee(top,list);
            //top节点要把totalFee也进行调差，这样后面调用dealTotalFee处理计算差额时不会出错，变电站不需要这一步，因为它把top放到list里，在循环调整金额时top节点也会调整
            top.setTotalFee(DoubleUtil.add(top.getTotalFee(),ftFee));
            top.setNodes(list);
        }
        //分摊依据字段和
        Double totalMount = 0.00;
        //分摊依据字段值
        Double theMount = 0.00;
        if(code.equals(BashInfo.TZ_ZJGC)){
            totalMount = top.getZjgcFee();
        }else if(code.equals(BashInfo.TZ_RGF)){
            totalMount = top.getOriginalRgCost();
        }else if(code.equals(BashInfo.TZ_CBRCG)){
            totalMount = top.getOriginalCbrCost();
        }else if(code.equals(BashInfo.TZ_FBRCG)){
            totalMount = top.getOriginalFbrCost();
        }else if(code.equals(BashInfo.TZ_JX)){
            totalMount = top.getOriginalJxCost();
        }else if(code.equals(BashInfo.TZ_CSF1)){
            totalMount = top.getMeasuresFee1();
        }else if(code.equals(BashInfo.TZ_CSF2)){
            totalMount = top.getMeasuresFee2();
        }else if(code.equals(BashInfo.TZ_CSF3)){
            totalMount = top.getMeasuresFee3();
        }else if(code.equals(BashInfo.TZ_QTF)){
            totalMount = top.getOtherCost();
        }else if(code.equals(BashInfo.TZ_GF)){
            totalMount = top.getFees();
        }else if(code.equals(BashInfo.TZ_SF)){
            totalMount = top.getTex();
        }
        if(totalMount==null||totalMount==0.00){
            DetailFeeUtil.avgShareToTotal(top,ftFee);
            DetailFeeUtil.dealTotalFee(top);
        }else{
            for(JkxlDetailFee fee:list){
                if(code.equals(BashInfo.TZ_ZJGC)){
                    theMount = fee.getZjgcFee();
                }else if(code.equals(BashInfo.TZ_RGF)){
                    theMount = fee.getOriginalRgCost();
                }else if(code.equals(BashInfo.TZ_CBRCG)){
                    theMount = fee.getOriginalCbrCost();
                }else if(code.equals(BashInfo.TZ_FBRCG)){
                    theMount = fee.getOriginalFbrCost();
                }else if(code.equals(BashInfo.TZ_JX)){
                    theMount = fee.getOriginalJxCost();
                }else if(code.equals(BashInfo.TZ_CSF1)){
                    theMount = fee.getMeasuresFee1();
                }else if(code.equals(BashInfo.TZ_CSF2)){
                    theMount = fee.getMeasuresFee2();
                }else if(code.equals(BashInfo.TZ_CSF3)){
                    theMount = fee.getMeasuresFee3();
                }else if(code.equals(BashInfo.TZ_QTF)){
                    theMount = fee.getOtherCost();
                }else if(code.equals(BashInfo.TZ_GF)){
                    theMount = fee.getFees();
                }else if(code.equals(BashInfo.TZ_SF)){
                    theMount = fee.getTex();
                }
                double rate = theMount/totalMount;
                double tzFee = DoubleUtil.round2(ftFee*rate);
                fee.setTotalFee(DoubleUtil.add(fee.getTotalFee(),tzFee));
            }
            DetailFeeUtil.dealTotalFee(top);
        }
        overheadLineMapper.updateList(list);
        //更新未识别费用表
        JkxlAdjustFee jkxlAdjustFee = new JkxlAdjustFee();
        jkxlAdjustFee.setType(type);
        jkxlAdjustFee.setId(id);
        feeAdjustService.updateFee(jkxlAdjustFee);
    }

    /**
     * 处理单独调整
     * @param ftData
     */
    @Transactional(readOnly = false)
    public void adjustAllocateNode(FtData ftData){
        String id = ftData.getId();
        String state  = ftData.getState();  // 1:增加 2:减少
        String data = ftData.getTableData();   //表格数据JOSN字符串
        String row = data.replaceAll("&quot;","\\\"");
        List<DdtzData> tzList = (List<DdtzData>) JSONArray.parseArray(row, DdtzData.class);
        for(DdtzData ddtzData: tzList){
            double tzFee = Double.valueOf(ddtzData.getFee());
            if(state.equals("2")){
                tzFee = -tzFee;
            }
            String wbsCode = ddtzData.getWbsCode();
            JkxlDetailFee fee = overheadLineMapper.getData(id,wbsCode);
            if(fee==null){
                fee = overheadLineMapper.getElement(wbsCode);
            }
//            如果调整的节点类型是其他费用，不管项目类型逐级调整，若是本体费用，变电站逐级调整，架空和电缆调整单个节点
            if(fee.getFeeType().equals("0")){
                saveLayerFee(id,wbsCode,"0",tzFee);
            }else if(fee.getFeeType().equals("1")&&fee.getProjectType().equals("3")){
                saveLayerFee(id,wbsCode,"1",tzFee);
            }else if(fee.getFeeType().equals("1")&&(fee.getProjectType().equals("1")||fee.getProjectType().equals("2"))){
                if(fee.getId()==null){
                    fee.setTotalFee(tzFee);
                    overheadLineMapper.save(fee);
                }else{
                    fee.setTotalFee(DoubleUtil.add(fee.getTotalFee(),tzFee));
                    overheadLineMapper.updateData(fee);
                }
            }
        }
        //更新未识别费用表
        JkxlAdjustFee jkxlAdjustFee = new JkxlAdjustFee();
        jkxlAdjustFee.setType(ftData.getType());
        jkxlAdjustFee.setId(id);
        feeAdjustService.updateFee(jkxlAdjustFee);

    }

    /**
     * 找到费用类型为feeType的节点，以及同样类型的所有父节点，逐层调整totalFee
     * @param id        结算书id
     * @param wbsCode   某其他费用节点编码
     * @param wbsCode   费用类型
     * @param mount     调整金额
     */
    @Transactional(readOnly = false)
    public void saveLayerFee(String id,String wbsCode,String feeType,double mount){
        if(StringUtils.isBlank(id)||StringUtils.isBlank(wbsCode)||mount==0.00){
            return;
        }
        List<JkxlDetailFee> list = new ArrayList<>();
        getFeeTypeParents(id,wbsCode,feeType,list);
        if(list.size()>0){
            for(JkxlDetailFee fee:list){
                if(fee.getId()==null){
                    fee.setId(id);
                    fee.setTotalFee(mount);
                    overheadLineMapper.save(fee);
                }else{
                    fee.setTotalFee(DoubleUtil.add(fee.getTotalFee(),mount));
                    overheadLineMapper.updateData(fee);
                }
            }
        }
    }


    /**
     * 找到一个节点的 某种费用类型的所有父节点
     * 此方法在其他费用新增，调整，和变电站本体的单个新增调整时用。
     * 这样做有效的前提是 我们在保存变电站结算书数据时，本体部分保存了feeType为1的节点。不适用于其他项目类型的本体部分原因是他们还有一个
     * “本体工程”节点feeType为1，但是我们在保存结算书数据时不保存这个节点
     * 而其他费用节点所有项目类型都保存额feeType为0的节点
     *
     * 注意：如果该项目在某个父节点上没有保存数据，则到wbs节点库中找到该父节点，这时候该父节点id为空
     * @param id        结算书id
     * @param wbsCode   节点编码
     * @param feeType   费用类型
     * @param list      节点list
     */
    private void getFeeTypeParents(String id,String wbsCode,String feeType,List<JkxlDetailFee> list){
        if(StringUtils.isBlank(id)||StringUtils.isBlank(wbsCode)){
            return;
        }
        JkxlDetailFee fee = overheadLineMapper.getData(id,wbsCode);
        if(fee==null){
            fee = overheadLineMapper.getElement(wbsCode);
        }
        if(fee.getFeeType().equals(feeType)){
            list.add(fee);
            getFeeTypeParents(id,fee.getParentNode(),feeType,list);
        }

    }



    /**
     * 架空线路保存
     * @param file
     * @param wbsItem
     * @throws IOException
     * @throws InvalidFormatException
     */
    @Transactional(readOnly = false)
    public void saveJk(MultipartFile file,WbsItem wbsItem) throws IOException, InvalidFormatException {
        String id = wbsItem.getId();
        WbsItem wbsItem1 = itemService.get(id);
        if(wbsItem1 == null){
            wbsItem.setDelFlag("0");
            //保存项目
            itemService.save(wbsItem);
        }
        //读取表格
        ImportExcel total = new ImportExcel(file,1,new String[]{BashInfo.SHEET_HZB,BashInfo.SHEET_HZB1,BashInfo.SHEET_HZB2,BashInfo.SHEET_HZB3},true );  //工程项目竣工结算汇总表
        ImportExcel importExcel = new ImportExcel(file,1,BashInfo.SHEET_FBFXJJ,true);  //分部分项工程量清单计价表
        ImportExcel csqd = new ImportExcel(file,1,new String[]{BashInfo.SHEET_CSB,BashInfo.SHEET_CSB1},true);  // 措施项目清单计价表（一）
        ImportExcel csqd2 = new ImportExcel(file,1,BashInfo.SHEET_CSB2,false);  // 措施项目清单计价表（二）
        ImportExcel gf = new ImportExcel(file,1,BashInfo.SHEET_GFQD,true);  //规费项目清单计价表
        ImportExcel qt = new ImportExcel(file,1,BashInfo.SHEET_QTFY,true);  //其他项目清单计价表
        ImportExcel gzhz = new ImportExcel(file,1,new String[]{BashInfo.SHEET_GCHZ,BashInfo.SHEET_GCHZ1},true );   //架空线路工程费用汇总
        ImportExcel cbrsb = new ImportExcel(file,1,new String[]{BashInfo.SHEET_TBRCG,BashInfo.SHEET_CBRCG},false);//承包人采购设备计价表
//        获取《工程项目竣工结算汇总表》数据
        double texTotal = ExcelUtil.getTex(total); //获取税金
        double btex = BashInfo.getBtex(total);  //获取补税额
        double gcjsFee = BashInfo.getGcjsFee(total);  //获取工程结算价
//        获取《架空线路工程费用汇总》中各分部分项的合计费用和直接工程费
        double[] fbfxFees = ExcelUtil.getDoubleArray(gzhz,BashInfo.HZ_ALL,1,2); //合计即分部分项
        double[] zjgcFees = ExcelUtil.getDoubleArray(gzhz,BashInfo.HZ_ALL,1,3);  //直接工程费
        double fbfxTotal = ExcelUtil.getDouble(gzhz,BashInfo.QT_HJ,0,2); //分部分项合计
        double zjgcTotal = ExcelUtil.getDouble(gzhz,BashInfo.QT_HJ,0,3);//直接工程费合计
//        获取《分部分项工程量清单计价表》详细费用数据
        double[][] originalFees = ExcelUtil.getDoubleArrays(importExcel,BashInfo.MX_ALL,2, new int[]{13, 14, 15, 17}); //人工费、承包人采购、发包人采购、机械费
        double rgTotal = ExcelUtil.getDouble(importExcel,BashInfo.PROJECTNAME,2,13);  //人工费合计金额
        double fbrTotal = ExcelUtil.getDouble(importExcel,BashInfo.PROJECTNAME,2,15); //发包人采购计价表合计金额
//        从《 措施项目清单计价表（一）》《（二）》获取措施费
        double[] csFees = BashInfo.getCsFees(csqd); //[0]临时设施+安全文明施工 [1]剩下
//        double[] csRate = BashInfo.getCsRate(csqd); //获取措施费率
        double cs2Total =  0.0;  //措施费二合计
        if(ImportExcel.getSheet(csqd2) != null){
            cs2Total = SubstationInfo.getCsJzFee(csqd2);
        }
//        获取《规费项目清单计价表》合计
        double gfTotal = BashInfo.getJkGf(gf);
//        获取《承包人采购设备计价表》合计
        double cbrTotal =  0.0; //承包人采购设备费
        if(ImportExcel.getSheet(cbrsb) != null){
            cbrTotal = ExcelUtil.getDouble(cbrsb,BashInfo.QT_HJ,0,6);
        }
//        获取《其他项目清单计价表》中其他费用和
        double qtTotal =  ExcelUtil.getDouble(qt,BashInfo.QT_HJ,0,2);//获取其他项目费合计
        double[] qtFees = BashInfo.getQtFees(qt);  //[0]为“施工场地租用费”，[1]为“招标代理”费用，[2]为剩余其他费用
//        根据安全文明施工费、费率、直接工程费合计计算进项税率
        double[] aqwmFees = BashInfo.getAqwfJzFee(csqd);  //安全文明施工费
        double zg = zjgcTotal-(aqwmFees[1]/(aqwmFees[0]/100));
        double jxsl = zg/(fbrTotal-zg); //进项税率
//        部分结算书的《架空线路工程费用汇总》和《分部分项工程量清单计价表》中多出措施项目的分部分项，处理方法是将该部分数据放到“辅助工程”中
        double csxmHj = ExcelUtil.getDouble(gzhz,BashInfo.HZ_CH,1,2);//措施项目合计
        double csxmZjgc = ExcelUtil.getDouble(gzhz,BashInfo.HZ_CH,1,3);//措施项目直接工程费
        double[] csxmFees = ExcelUtil.getDoubleArray(importExcel,BashInfo.CS,2,new int[]{13, 14, 15, 17});//措施项目费
        fbfxFees[5] = DoubleUtil.add(fbfxFees[5],csxmHj);
        zjgcFees[5] = DoubleUtil.add(zjgcFees[5], csxmZjgc);
        originalFees[5][0] = DoubleUtil.add(originalFees[5][0],csxmFees[0]);
        originalFees[5][1] = DoubleUtil.add(originalFees[5][1],csxmFees[1]);
        originalFees[5][2] = DoubleUtil.add(originalFees[5][2],csxmFees[2]);
        originalFees[5][3] = DoubleUtil.add(originalFees[5][3],csxmFees[3]);
        //数据核验数据
        List<Settlement> settlementList =  SettementUtil.getComparison(total);
        double[] divisiontotal=ExcelUtil.getDoubleArray(importExcel,BashInfo.PROJECTNAME,2,new int[]{12,15});//《分部分项工程量清单计价表》中分部分项合计与发包人合计
        List<Settlement> settlements=SettementUtil.getComparison(settlementList,fbfxTotal,divisiontotal,csFees[0]+csFees[1],cs2Total,qtTotal,gfTotal,cbrTotal);
        Boolean f = SettementUtil.getJgjsDeveloper(settlementList);
        settlementService.save(settlements,id);
        double csShare = 0.00;
        double gfShare = 0.00;
        double qtShare = 0.00;
        double texShare = 0.00;
        //将本体工程费用明细存入集合
        List<JkxlDetailFee> list = new ArrayList<>();
        for(int i=0;i<fbfxFees.length;i++){
            JkxlDetailFee jkxlDetailFee = new JkxlDetailFee();
            double fbfxCost = fbfxFees[i]; //原合计
            double zjgcCost = zjgcFees[i];//直接工程费
            double originalRgCost = originalFees[i][0];// 原人工费
            double originalCbrCost = originalFees[i][1];  //原承包人采购
            double originalFbrCost = originalFees[i][2];  //原发包人采购
            double originalJxrCost = originalFees[i][3];   //原机械费
            double fbfxRate = fbfxCost/fbfxTotal;   //分部分项费用占比
            double rgRate = originalRgCost/rgTotal; //人工费占比
            double meas1Rate = (zjgcCost-originalFbrCost*jxsl/(1+jxsl))/(zjgcTotal-fbrTotal*jxsl/(1+jxsl));//(分部分项直接工程费-分部分项进项税额)占(直接工程费合计-甲供材合计进项税额)比例
            double measuresFee1 = DoubleUtil.round2(csFees[0] * meas1Rate); //措施费一（1）即临时设施+安全文明施工
            double measuresFee2 = DoubleUtil.round2(csFees[1] * rgRate);
            double measuresFee3 = DoubleUtil.round2(cs2Total*fbfxRate); //措施费二
            double qtCost = DoubleUtil.round2(qtFees[2]*fbfxRate);  //除场地租用费、招标代理费外费用根据分部分项合计费用所占比例分摊
            double indepQt = DoubleUtil.round2((qtFees[0]+qtFees[1])*fbfxRate);
            double gfCost = DoubleUtil.round2(gfTotal*rgRate);  //规费
            double cbrSbCost = DoubleUtil.round2(cbrTotal*fbfxRate);//承包人设备采购费根据分部分项进行分摊
            double totalCost = 0.00;  //合计
            double payable = 0.00;//应纳税金额
            if(f){
                payable = DoubleUtil.add(fbfxTotal,csFees[0],csFees[1],cs2Total,qtTotal,gfTotal,cbrTotal-fbrTotal);
                totalCost = DoubleUtil.add(fbfxCost,measuresFee1,measuresFee2,measuresFee3,gfCost,qtCost,cbrSbCost,-originalFbrCost);
            } else {
                payable = DoubleUtil.add(fbfxTotal,csFees[0],csFees[1],cs2Total,qtTotal,gfTotal,cbrTotal);
                totalCost = DoubleUtil.add(fbfxCost,measuresFee1,measuresFee2,measuresFee3,gfCost,qtCost,cbrSbCost); //合计
            }
            double texCost = DoubleUtil.round2((texTotal+btex)*(totalCost+indepQt)/payable); //税金
            totalCost = DoubleUtil.add(totalCost,texCost);
            csShare = DoubleUtil.add(csShare,measuresFee1,measuresFee2,measuresFee3);
            qtShare = DoubleUtil.add(qtShare,qtCost);
            gfShare = DoubleUtil.add(gfShare,gfCost);
            texShare = DoubleUtil.add(texShare,texCost);
            jkxlDetailFee.setOriginalTotalCost(fbfxCost);
            jkxlDetailFee.setOriginalRgCost(originalRgCost);
            jkxlDetailFee.setOriginalCbrCost(originalCbrCost);
            jkxlDetailFee.setOriginalFbrCost(originalFbrCost);
            jkxlDetailFee.setOriginalJxCost(originalJxrCost);
            jkxlDetailFee.setMeasuresFee1(measuresFee1);
            jkxlDetailFee.setMeasuresFee2(measuresFee2);
            jkxlDetailFee.setMeasuresFee3(measuresFee3);
            jkxlDetailFee.setFees(gfCost);
            jkxlDetailFee.setOtherCost(qtCost);
            jkxlDetailFee.setTex(texCost);
            jkxlDetailFee.setTotalFee(totalCost);
            jkxlDetailFee.setZjgcFee(zjgcCost);
            jkxlDetailFee.setWbsCode(BashInfo.WBSIDS[i]);
            jkxlDetailFee.setId(id);
            list.add(jkxlDetailFee);
        }

        double csDiff =  DoubleUtil.sub(DoubleUtil.add(csFees[0],csFees[1],cs2Total),csShare);
        double qtDiff = DoubleUtil.sub(qtFees[2],qtShare);
        double gfDiff = DoubleUtil.sub(gfTotal,gfShare);
        double texDiff = DoubleUtil.sub(DoubleUtil.add(texTotal,btex),texShare);
        for(JkxlDetailFee fee:list){
            if(fee.getTotalFee()!=null&&fee.getTotalFee()!=0.00){
                fee.setMeasuresFee2(DoubleUtil.add(fee.getMeasuresFee2(),csDiff));
                fee.setOtherCost(DoubleUtil.add(fee.getOtherCost(),qtDiff));
                fee.setFees(DoubleUtil.add(fee.getFees(),gfDiff));
                fee.setTex(DoubleUtil.add(fee.getTex(),texDiff));
                fee.setTotalFee(DoubleUtil.add(fee.getTotalFee(),csDiff,qtDiff,gfDiff,texDiff));
                break;
            }
        }
        //获取未识别费用
        List<JkxlAdjustFee> wsbFees = BashInfo.getWsbFee(total);
        double wsbfee = 0.00;
        if(wsbFees != null && wsbFees.size()>0){
            for(JkxlAdjustFee jkxlAdjustFee : wsbFees){
//                取未识别费用合计,这里要处理一种情况，就是有些金额是应该减掉的，但是在结算书里是正数
                double fee = jkxlAdjustFee.getFee();
                if(fee>0&&(jkxlAdjustFee.getType().contains("减") || jkxlAdjustFee.getType().contains("扣") || jkxlAdjustFee.getType().contains("下浮"))){
                    fee = fee* -1;
                }
                wsbfee = DoubleUtil.add(wsbfee,fee);
                jkxlAdjustFee.setId(id);
                jkxlAdjustFee.setFee(Math.abs(fee));
                feeAdjustService.save(jkxlAdjustFee);
            }
        }
        //计算差额
        double tz = 0.00; //总合计金额
        for(JkxlDetailFee jkxlDetailFee : list){
            tz = DoubleUtil.add(tz,jkxlDetailFee.getTotalFee());
        }
        double gxFee = Math.abs(DoubleUtil.add(gcjsFee,-tz,-qtFees[0],-qtFees[1],-wsbfee));
        //将其他费用存入集合
        saveLayerFee(id,"34120000","0",qtFees[0]);
        saveLayerFee(id,"34220000","0",qtFees[1]);
        //保存费用明细
        overheadLineMapper.saveList(list);
        //保存未识别费用
        if(gxFee!=0.00){
            JkxlAdjustFee jkxlAdjustFee2 = new JkxlAdjustFee();
            jkxlAdjustFee2.setId(id);
            jkxlAdjustFee2.setType(BashInfo.CE);
            jkxlAdjustFee2.setFee(gxFee);
            feeAdjustService.save(jkxlAdjustFee2);
        }
    }

    /**
     * 电缆线路保存
     * @param file
     * @param wbsItem
     * @throws IOException
     * @throws InvalidFormatException
     */
    @Transactional(readOnly = false)
    public void saveDl(MultipartFile file,WbsItem wbsItem) throws IOException, InvalidFormatException {
        String id = wbsItem.getId();
        WbsItem wbsItem1 = itemService.get(id);
        if(wbsItem1 == null){
            wbsItem.setDelFlag("0");
            //保存项目
            itemService.save(wbsItem);
        }
        //读取表格
        ImportExcel total = new ImportExcel(file,1,new String[]{BashInfo.SHEET_HZB,BashInfo.SHEET_HZB1,BashInfo.SHEET_HZB2,BashInfo.SHEET_HZB3},true );  //工程项目竣工结算汇总表
        ImportExcel csqd = new ImportExcel(file,1,new String[]{BashInfo.SHEET_CSB,BashInfo.SHEET_CSB1},true);  // 措施项目清单计价表（一）
        ImportExcel csqd2 = new ImportExcel(file,1,BashInfo.SHEET_CSB2,false);  // 措施项目清单计价表（二）
        ImportExcel gf = new ImportExcel(file,1,BashInfo.SHEET_GFQD,true);  //规费项目清单计价表
        ImportExcel qt = new ImportExcel(file,1,BashInfo.SHEET_QTFY,true);  //其他项目清单计价表
        ImportExcel cbrsb = new ImportExcel(file,1,new String[]{BashInfo.SHEET_TBRCG,BashInfo.SHEET_CBRCG},false);//承包人采购设备计价表
//        从《工程项目竣工结算汇总表》获取数据
        double jzgc = ExcelUtil.getDouble(total,new String[]{BashInfo.HZ_JZGC,BashInfo.HZ_JZGCF},1,2);//汇总表建筑金额
        double azgc = ExcelUtil.getDouble(total,new String[]{BashInfo.HZ_AZGC,BashInfo.HZ_AZGCF},1,2);//汇总表安装金额
        double fbfxFee = ExcelUtil.getDouble(total,BashInfo.JS_FBFX,1,2);  //分部分项工程费
        double texTotal = ExcelUtil.getTex(total);   //税金合计
        double btex = BashInfo.getBtex(total); //获取补税额
        double fbrTotal = ExcelUtil.getFbr(total); //获取发包人采购设备费
        double gcjsFee = BashInfo.getGcjsFee(total);  //获取工程结算价
//        从《 措施项目清单计价表（一）》《（二）》获取措施费
        double jzcsTotal = BashInfo.getJzCs(csqd); //建筑措施费合计
        double azcsTotal = BashInfo.getAzCs(csqd);  //安装措施费合计
        double csTotal = jzcsTotal+azcsTotal;//获取措施费清单计价表一
        double[] azcsFees = BashInfo.getAzCsFees(csqd);// 安装措施费 [0]临时设置+安全文明  [1]剩余和
        double[] jzcsFees = BashInfo.getJzCsFees(csqd); //建筑措施费 [0]临时设置+安全文明  [1]剩余和
        double jzcs2Total = 0.00; //措施二建筑
        double azcs2Total = 0.00;  //措施二安装
        double cs2Total=0.00;  //
        if(ImportExcel.getSheet(csqd2) != null){
            jzcs2Total = SubstationInfo.getCs2JzFee(csqd2);
            azcs2Total = SubstationInfo.getCs2AzFee(csqd2);
            cs2Total=jzcs2Total+azcs2Total;
        }
//        《承包人采购设备计价表》获取数据
        double cbrJzFee = 0.00; //投标人建筑设备费
        double cbrAzFee = 0.00; //投标人安装设备费
        double cbrTotal = 0.00; //投标人合计
        if(ImportExcel.getSheet(cbrsb) != null){
            cbrJzFee = BashInfo.getFbrJzFee(cbrsb);
            cbrAzFee = BashInfo.getFbrAzFee(cbrsb);
            cbrTotal = DoubleUtil.add(cbrJzFee,cbrAzFee);
        }
//        《其他项目清单计价表》获取数据
        double qtTotal =  ExcelUtil.getDouble(qt,BashInfo.QT_HJ,0,2);//获取其他项目费合计
        double[] qtFees = BashInfo.getQtFees(qt);  //[0]为“施工场地租用费”，[1]为“招标代理”费用，[2]为剩余其他费用
//        从《规费项目清单计价表》获取数据
        double gfTotal =  ExcelUtil.getDouble(gf,BashInfo.QT_HJ,0,4);//获取规费合计
        double azGf = BashInfo.getAzGf(gf); //获取安装规费
        double jzGf = BashInfo.getJzGf(gf);
//        准备数据校验
        List<Settlement> settlementList =  SettementUtil.getComparison(total);
        Boolean f = SettementUtil.getJgjsDeveloper(settlementList);
        double[] fbfxTotalArr = {0.00,0.00};
        double[] fbfxFbrTotalArr = {0.00,0.00,0.00,0.00};
        double csJzShare = 0.00;
        double csAzShare = 0.00;
        double gfJzShare = 0.00;
        double gfAzShare = 0.00;
        double cs2Share = 0.00;
        double qtShare = 0.00;
        double texShare = 0.00;
        //应纳税金额
        double payableTotal = 0.00;
        if(f){
            payableTotal = DoubleUtil.add(fbfxFee,csTotal,cs2Total,cbrTotal,qtTotal,gfTotal,-fbrTotal);
        }else{
            payableTotal = DoubleUtil.add(fbfxFee,csTotal,cs2Total,cbrTotal,qtTotal,gfTotal);
        }

        if(jzgc==0.00 && azgc==0.00){
            throw new RuntimeException(BashInfo.NUMBERERROR);
        }
        List<JkxlDetailFee> jzList = new ArrayList<>();
        List<JkxlDetailFee> azList = new ArrayList<>();
        if(jzgc !=0.00){
            ImportExcel jzqd = new ImportExcel(file,1,BashInfo.SHEET_JZFBFXJJ,true);  //建筑分部分项工程量清单计价表
            ImportExcel jzhz = new ImportExcel(file,1,BashInfo.SHEET_JZHZB,true );   //电缆线路建筑工程费用汇总表
//           《电缆线路建筑工程费用汇总表》数据
            double fbfxJzTotal = ExcelUtil.getDouble(jzhz,BashInfo.QT_HJ,0,2);//建筑分布分项工程量清单计价表合计
            double zjgcJzTotal = ExcelUtil.getDouble(jzhz,BashInfo.QT_HJ,0,3);//建筑直接工程费合计
            double[] fbfxJzFees = ExcelUtil.getDoubleArray(jzhz,BashInfo.HZ_JZ,1,2); //获取建筑合计费
            double[] zjgcJzFees = ExcelUtil.getDoubleArray(jzhz,BashInfo.HZ_JZ,1,3);  //获取建筑直接工程费
//            《建筑分部分项工程量清单计价表》数据
            double[][] originalJzFees = ExcelUtil.getDoubleArrays(jzqd,BashInfo.MX_JZ,2, new int[]{13, 14, 15, 17}); //人工费、承包人采购、发包人采购、机械费
            double fbrJzTotal = ExcelUtil.getDouble(jzqd,BashInfo.DLJZ,2,15);
//            准备结算书校验数据
            fbfxTotalArr[0] = fbfxJzTotal;
            double[] fbfxFbrTotal=ExcelUtil.getDoubleArray(jzqd,BashInfo.DLJZ,2,new int[]{12,15});//《建筑分部分项工程量清单计价表》分部分项、发包人采购合计
            fbfxFbrTotalArr[0] = fbfxFbrTotal[0];
            fbfxFbrTotalArr[1] = fbfxFbrTotal[1];
//            计算进项税率
            double[] aqwmJzFees = BashInfo.getAqwfJzFee(csqd);  //建筑安全文明施工费
            double jxslJz = 0.00;
            if(fbrJzTotal != 0.00){
                double zgJz = zjgcJzTotal-(aqwmJzFees[1]/(aqwmJzFees[0]/100));
                jxslJz = zgJz/(fbrJzTotal-zgJz); //建筑进项税率
            }
            //处理分部分项中多出的措施项目
            double csxmHj = ExcelUtil.getDouble(jzhz,BashInfo.HZ_CH,1,2);//措施项目合计
            double csxmZjgc = ExcelUtil.getDouble(jzhz,BashInfo.HZ_CH,1,3);//措施项目直接工程费
            double[] csxmFees = ExcelUtil.getDoubleArray(jzqd,BashInfo.MX_JZCSXM,2,new int[]{13, 14, 15, 17});//措施项目费
            fbfxJzFees[2] = DoubleUtil.add(fbfxJzFees[2],csxmHj);
            zjgcJzFees[2] = DoubleUtil.add(zjgcJzFees[2],csxmZjgc);
            originalJzFees[2][0] = DoubleUtil.add(originalJzFees[2][0],csxmFees[0]);
            originalJzFees[2][1] = DoubleUtil.add(originalJzFees[2][1],csxmFees[1]);
            originalJzFees[2][2] = DoubleUtil.add(originalJzFees[2][2],csxmFees[2]);
            originalJzFees[2][3] = DoubleUtil.add(originalJzFees[2][3],csxmFees[3]);
            for(int i=0;i<fbfxJzFees.length;i++){
                JkxlDetailFee jkxlDetailFee = new JkxlDetailFee();
                double originalTotalCost = fbfxJzFees[i]; //原合计
                double zjgcJzFee = zjgcJzFees[i];//直接工程费
                double originalRgCost = originalJzFees[i][0];// 原人工费
                double originalCbrCost = originalJzFees[i][1];  //原承包人采购
                double originalFbrCost = originalJzFees[i][2];  //原发包人采购
                double originalJxrCost = originalJzFees[i][3];   //原机械费
                double fbfxRate = originalTotalCost/fbfxJzTotal; //分部分项占建筑分部分项比例
                double fbfxTotalRate = originalTotalCost/fbfxFee;//分部分项费用占（建筑分部分项和+安装分部分项和）比例
//                (分部分项直接工程费-分部分项进项税额)占(建筑直接工程费合计-建筑甲供材合计进项税额)比例
                double measureRate = (zjgcJzFee-originalFbrCost/(1+jxslJz)*jxslJz)/(zjgcJzTotal-fbrJzTotal/(1+jxslJz)*jxslJz);
                double measuresFee1 = DoubleUtil.round2(jzcsFees[0]*measureRate); //措施费一（1）
                double measuresFee2 = DoubleUtil.round2(jzcsFees[1]*measureRate);   //措施费一（2）
                double measuresFee3 = DoubleUtil.round2(jzcs2Total*fbfxRate); //措施费二
                double gfCost = DoubleUtil.round2(jzGf*measureRate);
                double qtCost = DoubleUtil.round2(qtFees[2]*fbfxTotalRate);
                double indepQt = DoubleUtil.round2((qtFees[0]+qtFees[1])*fbfxTotalRate);
                double cbrSbCost = DoubleUtil.round2(cbrJzFee*fbfxRate);
                double totalCost = 0.00;
                if(f){
                    totalCost = DoubleUtil.add(originalTotalCost,measuresFee1,measuresFee2,measuresFee3,gfCost,qtCost,cbrSbCost,-originalFbrCost);
                } else {
                    totalCost = DoubleUtil.add(originalTotalCost,measuresFee1,measuresFee2,measuresFee3,gfCost,qtCost,cbrSbCost); //合计
                }
                double texCost = DoubleUtil.round2((texTotal+btex)*(totalCost+indepQt)/payableTotal);
                totalCost = DoubleUtil.add(totalCost,texCost);
                csJzShare = DoubleUtil.add(csJzShare,measuresFee1,measuresFee2);
                cs2Share = DoubleUtil.add(cs2Share,measuresFee3);
                gfJzShare = DoubleUtil.add(gfJzShare,gfCost);
                qtShare = DoubleUtil.add(qtShare,qtCost);
                texShare = DoubleUtil.add(texShare,texCost);
                jkxlDetailFee.setOriginalTotalCost(originalTotalCost);
                jkxlDetailFee.setOriginalRgCost(originalRgCost);
                jkxlDetailFee.setOriginalCbrCost(originalCbrCost);
                jkxlDetailFee.setOriginalFbrCost(originalFbrCost);
                jkxlDetailFee.setOriginalJxCost(originalJxrCost);
                jkxlDetailFee.setMeasuresFee1(measuresFee1);
                jkxlDetailFee.setMeasuresFee2(measuresFee2);
                jkxlDetailFee.setMeasuresFee3(measuresFee3);
                jkxlDetailFee.setFees(gfCost);
                jkxlDetailFee.setOtherCost(qtCost);
                jkxlDetailFee.setTex(texCost);
                jkxlDetailFee.setTotalFee(totalCost);
                jkxlDetailFee.setZjgcFee(zjgcJzFee);
                jkxlDetailFee.setWbsCode(BashInfo.JZWBSIDS [i]);
                jkxlDetailFee.setId(id);
                jzList.add(jkxlDetailFee);
            }
        }
        if(azgc != 0.00){
            ImportExcel azqd = new ImportExcel(file,1,BashInfo.SHEET_AZFBFXJJ,true);  //安装分部分项工程量清单计价表
            ImportExcel azhz = new ImportExcel(file,1,BashInfo.SHEET_HZHZB ,true);   //电缆线路安装工程费用汇总表
//           《电缆线路建筑工程费用汇总表》数据
            double fbfxAzTotal = ExcelUtil.getDouble(azhz,BashInfo.QT_HJ,0,2);//分部分项合计
            double zjgcAzTotal = ExcelUtil.getDouble(azhz,BashInfo.QT_HJ,0,3);// 直接工程费合计
            double[] fbfxAzFees = ExcelUtil.getDoubleArray(azhz,BashInfo.HZ_AZ,1,2); //分部分项费用
            double[] zjgcAzFees = ExcelUtil.getDoubleArray(azhz,BashInfo.HZ_AZ,1,3);  //直接工程费
            double rgAzTotal = ExcelUtil.getDouble(azhz,BashInfo.QT_HJ,0,4);//安装人工费费合计
//            《安装分部分项工程量清单计价表》数据
            double[][] originalAzFees = ExcelUtil.getDoubleArrays(azqd,BashInfo.MX_AZ,2, new int[]{13, 14, 15, 17});  //人工费、承包人采购、发包人采购、机械费
            double fbrAzTotal = ExcelUtil.getDouble(azqd,BashInfo.DLAZ,2,15);
//            准备结算书校验数据
            fbfxTotalArr[1] = fbfxAzTotal;
            double[] fbfxFbrTotal=ExcelUtil.getDoubleArray(azqd,BashInfo.DLAZ,2,new int[]{12,15});//《安装分部分项工程量清单计价表》分部分项、发包人采购合计
            fbfxFbrTotalArr[2] = fbfxFbrTotal[0];
            fbfxFbrTotalArr[3] = fbfxFbrTotal[1];
//            计算进项税率
            double[] aqwmAzFees = BashInfo.getAqwfAzFee(csqd);  //安装安全文明施工费
            double jxslAz = 0.00;
            if(fbrAzTotal != 0.00){
                double zgAz = zjgcAzTotal-(aqwmAzFees[1]/(aqwmAzFees[0]/100));
                jxslAz = zgAz/(fbrAzTotal-zgAz); //安装进项税率
            }
            //处理分部分项中多出的措施项目
            double csxmHj = ExcelUtil.getDouble(azhz,BashInfo.HZ_CH,1,2);//措施项目合计
            double csxmZjgc = ExcelUtil.getDouble(azhz,BashInfo.HZ_CH,1,3);//措施项目直接工程费
            double[] csxmFees = ExcelUtil.getDoubleArray(azqd,BashInfo.MX_AZCSXM,2,new int[]{13, 14, 15, 17});//措施项目费
            fbfxAzFees[5] = DoubleUtil.add(fbfxAzFees[5],csxmHj);
            zjgcAzFees[5] = DoubleUtil.add(zjgcAzFees[5],csxmZjgc);
            originalAzFees[5][0] = DoubleUtil.add(originalAzFees[5][0],csxmFees[0]);
            originalAzFees[5][1] = DoubleUtil.add(originalAzFees[5][1],csxmFees[1]);
            originalAzFees[5][2] = DoubleUtil.add(originalAzFees[5][2],csxmFees[2]);
            originalAzFees[5][3] = DoubleUtil.add(originalAzFees[5][3],csxmFees[3]);
            for(int i=0;i<fbfxAzFees.length;i++){
                JkxlDetailFee jkxlDetailFee = new JkxlDetailFee();
                double originalTotalCost = fbfxAzFees[i]; //原合计
                double zjgcAzFee = zjgcAzFees[i];//直接工程费
                double originalRgCost = originalAzFees[i][0];// 原人工费
                double originalCbrCost = originalAzFees[i][1];  //原承包人采购
                double originalFbrCost = originalAzFees[i][2];  //原发包人采购
                double originalJxrCost = originalAzFees[i][3];   //原机械费
                double fbfxRate = originalTotalCost/fbfxAzTotal; //分部分项占建筑分部分项比例
                double fbfxTotalRate = originalTotalCost/fbfxFee;//分部分项费用占（建筑分部分项和+安装分部分项和）比例
//                (分部分项直接工程费-分部分项进项税额)占(建筑直接工程费合计-建筑甲供材合计进项税额)比例
                double measureRate = (zjgcAzFee-originalFbrCost/(1+jxslAz)*jxslAz)/(zjgcAzTotal-fbrAzTotal/(1+jxslAz)*jxslAz);
                double rgRate = originalRgCost/rgAzTotal;
                double measuresFee1 = DoubleUtil.round2(azcsFees[0]*measureRate); //措施费一（1）
                double measuresFee2 = DoubleUtil.round2(azcsFees[1]*rgRate);   //措施费一（2）
                double measuresFee3 = DoubleUtil.round2(azcs2Total*fbfxRate); //措施费二
                double gfCost = DoubleUtil.round2(azGf*rgRate);
                double qtCost = DoubleUtil.round2(qtFees[2]*fbfxTotalRate);
                double indepQt = DoubleUtil.round2((qtFees[0]+qtFees[1])*fbfxTotalRate);
                double cbrSbCost = DoubleUtil.round2(cbrAzFee*fbfxRate);
                double totalCost = 0.00;
                if(f){
                    totalCost = DoubleUtil.add(originalTotalCost,measuresFee1,measuresFee2,measuresFee3,gfCost,qtCost,cbrSbCost,-originalFbrCost);
                } else {
                    totalCost = DoubleUtil.add(originalTotalCost,measuresFee1,measuresFee2,measuresFee3,gfCost,qtCost,cbrSbCost);
                }
                double texCost = DoubleUtil.round2((texTotal+btex)*(totalCost+indepQt)/payableTotal);
                totalCost = DoubleUtil.add(totalCost,texCost);
                csAzShare = DoubleUtil.add(csAzShare,measuresFee1,measuresFee2);
                cs2Share = DoubleUtil.add(cs2Share,measuresFee3);
                gfAzShare = DoubleUtil.add(gfAzShare,gfCost);
                qtShare = DoubleUtil.add(qtShare,qtCost);
                texShare = DoubleUtil.add(texShare,texCost);
                jkxlDetailFee.setOriginalTotalCost(originalTotalCost);
                jkxlDetailFee.setOriginalRgCost(originalRgCost);
                jkxlDetailFee.setOriginalCbrCost(originalCbrCost);
                jkxlDetailFee.setOriginalFbrCost(originalFbrCost);
                jkxlDetailFee.setOriginalJxCost(originalJxrCost);
                jkxlDetailFee.setMeasuresFee1(measuresFee1);
                jkxlDetailFee.setMeasuresFee2(measuresFee2);
                jkxlDetailFee.setMeasuresFee3(measuresFee3);
                jkxlDetailFee.setFees(gfCost);
                jkxlDetailFee.setOtherCost(qtCost);
                jkxlDetailFee.setTex(texCost);
                jkxlDetailFee.setTotalFee(totalCost);
                jkxlDetailFee.setZjgcFee(zjgcAzFee);
                jkxlDetailFee.setWbsCode(BashInfo.AZWBSIDS [i]);
                jkxlDetailFee.setId(id);
                azList.add(jkxlDetailFee);
            }
        }
//        将措施费、规费、其他费用、税金的分摊差额放到一个有值的本体工程上
        double csJzDiff =  DoubleUtil.sub(DoubleUtil.add(jzcsFees[0],jzcsFees[1]),csJzShare);
        double csAzDiff = DoubleUtil.sub(DoubleUtil.add(azcsFees[0],azcsFees[1]),csAzShare);
        double cs2Diff = DoubleUtil.sub(cs2Total,cs2Share);
        double qtDiff = DoubleUtil.sub(qtFees[2],qtShare);
        double gfJzDiff = DoubleUtil.sub(jzGf,gfJzShare);
        double gfAzDiff = DoubleUtil.sub(azGf,gfAzShare);
        double texDiff = DoubleUtil.sub(DoubleUtil.add(texTotal,btex),texShare);
        if(jzgc!=0.00){
            for(JkxlDetailFee fee:jzList){
                if(fee.getTotalFee()!=null&&fee.getTotalFee()!=0.00){
                    fee.setMeasuresFee2(DoubleUtil.add(fee.getMeasuresFee2(),csJzDiff));
                    fee.setMeasuresFee3(DoubleUtil.add(fee.getMeasuresFee3(),cs2Diff));
                    fee.setOtherCost(DoubleUtil.add(fee.getOtherCost(),qtDiff));
                    fee.setFees(DoubleUtil.add(fee.getFees(),gfJzDiff));
                    fee.setTex(DoubleUtil.add(fee.getTex(),texDiff));
                    fee.setTotalFee(DoubleUtil.add(fee.getTotalFee(),csJzDiff,cs2Diff,qtDiff,gfJzDiff,texDiff));
                    break;
                }
            }
        }
        if(azgc!=0.00){
            for(JkxlDetailFee fee:azList){
                if(fee.getTotalFee()!=null&&fee.getTotalFee()!=0.00){
                    fee.setMeasuresFee2(DoubleUtil.add(fee.getMeasuresFee2(),csAzDiff));
                    fee.setFees(DoubleUtil.add(fee.getFees(),gfAzDiff));
                    if(jzgc == 0.00){
                        fee.setMeasuresFee3(DoubleUtil.add(fee.getMeasuresFee3(),cs2Diff));
                        fee.setOtherCost(DoubleUtil.add(fee.getOtherCost(),qtDiff));
                        fee.setTex(DoubleUtil.add(fee.getTex(),texDiff));
                        fee.setTotalFee(DoubleUtil.add(fee.getTotalFee(),csAzDiff,cs2Diff,qtDiff,gfAzDiff,texDiff));
                    }else{
                        fee.setTotalFee(DoubleUtil.add(fee.getTotalFee(),csAzDiff,gfAzDiff));
                    }
                    break;
                }
            }
        }
//        处理数据交易
        List<Settlement> dlSettlements = SettementUtil.getDLComparison(settlementList,fbfxTotalArr,fbfxFbrTotalArr,csTotal,cs2Total,qtTotal,gfTotal,cbrTotal);
        //数据核验并保存数据库
        settlementService.save(dlSettlements,id);
//        wbs节点对象合并
        jzList.addAll(azList);

        //获取未识别费用
        List<JkxlAdjustFee> wsbFees = BashInfo.getWsbFee(total);
        double wsbfee = 0.00;
        if(wsbFees != null && wsbFees.size()>0){
            for(JkxlAdjustFee jkxlAdjustFee : wsbFees){
                //                取未识别费用合计,这里要处理一种情况，就是有些金额是应该减掉的，但是在结算书里是正数
                double fee = jkxlAdjustFee.getFee();
                if(fee>0&&(jkxlAdjustFee.getType().contains("减") || jkxlAdjustFee.getType().contains("扣") || jkxlAdjustFee.getType().contains("下浮"))){
                    fee = fee* -1;
                }
                wsbfee = DoubleUtil.add(wsbfee,fee);
                jkxlAdjustFee.setId(id);
                jkxlAdjustFee.setFee(Math.abs(fee));
                feeAdjustService.save(jkxlAdjustFee);
            }
        }
        //计算差额
        double tz = 0.00; //总合计金额
        for(JkxlDetailFee jkxlDetailFee : jzList){
            tz += jkxlDetailFee.getTotalFee();
        }
        double gxFee = Math.abs(DoubleUtil.add(gcjsFee,-tz,-qtFees[0],-qtFees[1],-wsbfee));
        //将其他费用存入集合
        saveLayerFee(id,"44120000","0",qtFees[0]);
        saveLayerFee(id,"44220000","0",qtFees[1]);

        overheadLineMapper.saveList(jzList);
        if(gxFee!=0.00){
            JkxlAdjustFee jkxlAdjustFee2 = new JkxlAdjustFee();
            jkxlAdjustFee2.setId(id);
            jkxlAdjustFee2.setType(BashInfo.CE);
            jkxlAdjustFee2.setFee(gxFee);
            feeAdjustService.save(jkxlAdjustFee2);
        }
    }


    /**
     * 获取导出数据
     * @param id
     * @param type
     * @return
     */
    public List<JkxlDetailFee> findList(String id,String type){
        List<JkxlDetailFee> oriList = overheadLineMapper.getDetails(id,null,null,null,null);
//        获取调整金额
        double integrityFee = 0.00; //诚信扣款
        double totalDeal = 0.00;//需处理税金与甲供材抵扣款和
        WbsItem wbsItem = itemService.get(id);
        if(wbsItem != null){
            if(wbsItem.getIntegrityFee() != null){
                integrityFee = wbsItem.getIntegrityFee() * -1;
            }
            totalDeal = DoubleUtil.add(wbsItem.getDecFund(),wbsItem.getTexDeal());
        }
        Double jz2 = 0.00;//调整后建筑费合计
        Double az2 = 0.00;//调整后安装费合计
        Double qtTotal = 0.00;
        //变电站导出
        if(type.equals(BashInfo.PROJECT_BDZ)) {
            List<JkxlDetailFee> list = new ArrayList<>();
            JkxlDetailFee topFee = null;  //顶级节点
            JkxlDetailFee bdzFee = null;;  //变电站项目节点
            JkxlDetailFee jzBodyFee = null; //建筑本体工程节点
            JkxlDetailFee azBodyFee = null; //安装本体工程节点
            JkxlDetailFee qtFee = null;    //其他费用顶级节点
            JkxlDetailFee midQtFee = null;  //诚信扣款处理节点上级节点
            JkxlDetailFee cxkkFee = null;   //诚信扣款处理节点
            for(JkxlDetailFee fee:oriList){
//                获取变电站项目“3”，费用类型为本体“1”，展示状态为展示“1”（这里剔除了本体中的设备购置费），totalFee不为空的节点
                if("3".equals(fee.getProjectType())&&"1".equals(fee.getFeeType())&&
                        "1".equals(fee.getState())&&fee.getTotalFee()!=null){
                    list.add(fee);
                }
                //找到变电站节点
                if(fee.getWbsCode().equals("10000000")){
                    topFee = fee;
                }
                if(fee.getWbsCode().equals("20000000")){
                    bdzFee = fee;
                }
                if(fee.getWbsCode().equals("21000000")){
                    jzBodyFee = fee;
                }
                if(fee.getWbsCode().equals("23000000")){
                    azBodyFee = fee;
                }
                if(fee.getWbsCode().equals("24000000")){
                    qtFee = fee;
                }
                if(fee.getWbsCode().equals("24200000")){
                    midQtFee = fee;
                }
                if(fee.getWbsCode().equals("24260000")){
                    cxkkFee = fee;
                }
            }
//            将变电站节点加入到list
            list.add(bdzFee);
            DetailFeeUtil.getTree(list);
            DetailFeeUtil.getGatherFeeOfTotal(bdzFee,bdzFee.getNodes());
            //处理税金和甲供材超供抵扣款
            if(totalDeal!=0.00){
                DetailFeeUtil.shareByTotal(bdzFee,-totalDeal);
                DetailFeeUtil.dealTotalFee(bdzFee);
            }
            jz2 = jzBodyFee.getTotalFee();
            az2 = azBodyFee.getTotalFee();
            qtTotal = DoubleUtil.add(qtFee.getTotalFee(),integrityFee);
            //处理一些特殊节点
            topFee.setJzFee(jz2);
            topFee.setAzFee(az2);
            topFee.setTotalFee(DoubleUtil.add(jz2,az2,qtTotal));
            topFee.setQtFee(qtTotal);
            bdzFee.setJzFee(jz2);
            bdzFee.setAzFee(az2);
            bdzFee.setQtFee(qtTotal);
            bdzFee.setTotalFee(DoubleUtil.add(jz2,az2,qtTotal));
            if(integrityFee!=0.0){
                cxkkFee.setTotalFee(integrityFee);
                midQtFee.setTotalFee(DoubleUtil.add(midQtFee.getTotalFee(),integrityFee));
                qtFee.setTotalFee(qtTotal);
            }
        }
        //电缆导出
        if(type.equals(BashInfo.PROJECT_DLXL)){
            List<JkxlDetailFee> bodyList = new ArrayList<>();
            JkxlDetailFee topFee = null;  //顶级节点
            JkxlDetailFee dlxlFee = null;  //电缆线路项目节点
            JkxlDetailFee bodyFee = null;  //本体工程节点
            JkxlDetailFee jzBodyFee = null; //建筑本体工程节点
            JkxlDetailFee azBodyFee = null; //安装本体工程节点
            JkxlDetailFee qtFee = null;    //其他费用顶级节点
            JkxlDetailFee midQtFee = null;  //诚信扣款处理节点上级节点
            JkxlDetailFee cxkkFee = null;   //诚信扣款处理节点
            for (JkxlDetailFee fee : oriList) {
                if("2".equals(fee.getProjectType())&&"1".equals(fee.getFeeType())&&"1".equals(fee.getState())){
                    bodyList.add(fee);
                }
                String wbsCode = fee.getWbsCode();
                if(wbsCode.equals("10000000")){
                    topFee = fee;
                }
                if(wbsCode.equals("40000000")){
                    dlxlFee = fee;
                }
                if(wbsCode.equals("49000000")){
                    bodyFee = fee;
                }
                if(wbsCode.equals("49100000")){
                    jzBodyFee = fee;
                }
                if(wbsCode.equals("49300000")){
                    azBodyFee = fee;
                }
                if(wbsCode.equals("44000000")){
                    qtFee = fee;
                }
                if(wbsCode.equals("44200000")){
                    midQtFee = fee;
                }
                if(wbsCode.equals("44260000")){
                    cxkkFee = fee;
                }
            }
            DetailFeeUtil.getTree(bodyList);
            DetailFeeUtil.getGatherFeeOfTotal(jzBodyFee,jzBodyFee.getNodes());
            DetailFeeUtil.getGatherFeeOfTotal(azBodyFee,azBodyFee.getNodes());
            DetailFeeUtil.getGatherFeeOfTotal(bodyFee,bodyFee.getNodes());
            //处理税金和甲供材超供抵扣款
            if(totalDeal!=0.00){
                DetailFeeUtil.shareByTotal(bodyFee,-totalDeal);
                DetailFeeUtil.dealTotalFee(bodyFee);
            }

            jz2 = jzBodyFee.getTotalFee();
            az2 = azBodyFee.getTotalFee();
            qtTotal = DoubleUtil.add(qtFee.getTotalFee(),integrityFee);
            //处理一些特殊节点
            topFee.setJzFee(jz2);
            topFee.setAzFee(az2);
            topFee.setTotalFee(DoubleUtil.add(jz2,az2,qtTotal));
            topFee.setQtFee(qtTotal);
            dlxlFee.setJzFee(jz2);
            dlxlFee.setAzFee(az2);
            dlxlFee.setTotalFee(DoubleUtil.add(jz2,az2,qtTotal));
            dlxlFee.setQtFee(qtTotal);
            bodyFee.setJzFee(jz2);
            bodyFee.setAzFee(az2);
            if(integrityFee!=0.0){
                cxkkFee.setTotalFee(integrityFee);
                midQtFee.setTotalFee(DoubleUtil.add(midQtFee.getTotalFee(),integrityFee));
                qtFee.setTotalFee(qtTotal);
            }
        }
        //架空导出
        if(type.equals(BashInfo.PROJECT_JKXL)) {
            List<JkxlDetailFee> bodyList = new ArrayList<>();
            JkxlDetailFee topFee = null;  //顶级节点
            JkxlDetailFee jkxlFee = null;  //架空线路项目节点
            JkxlDetailFee bodyFee = null;  //本体工程节点
            JkxlDetailFee qtFee = null;    //其他费用顶级节点
            JkxlDetailFee midQtFee = null;  //诚信扣款处理节点上级节点
            JkxlDetailFee cxkkFee = null;   //诚信扣款处理节点
            for (JkxlDetailFee fee : oriList) {
                //本体费用求和
                if("1".equals(fee.getProjectType())&&"1".equals(fee.getFeeType())){
                    bodyList.add(fee);
                }
                if(fee.getWbsCode().equals("10000000")){
                    topFee = fee;
                }
                if(fee.getWbsCode().equals("30000000")){
                    jkxlFee = fee;
                }
                if(fee.getWbsCode().equals("39000000")){
                    bodyFee = fee;
                }
                if(fee.getWbsCode().equals("34000000")){
                    qtFee = fee;
                }
                if(fee.getWbsCode().equals("34200000")){
                    midQtFee = fee;
                }
                if(fee.getWbsCode().equals("34260000")){
                    cxkkFee = fee;
                }
            }
            DetailFeeUtil.getTree(bodyList);
            DetailFeeUtil.getGatherFeeOfTotal(bodyFee,bodyFee.getNodes());
            //处理税金和甲供材超供抵扣款
            if(totalDeal!=0.00){
                DetailFeeUtil.shareByTotal(bodyFee,-totalDeal);
                DetailFeeUtil.dealTotalFee(bodyFee);
            }
            az2 = bodyFee.getTotalFee();
            qtTotal = DoubleUtil.add(qtFee.getTotalFee(),integrityFee);
            //处理几个特殊节点金额
            topFee.setTotalFee(DoubleUtil.add(az2,qtTotal));
            topFee.setAzFee(az2);
            topFee.setQtFee(qtTotal);
            jkxlFee.setTotalFee(DoubleUtil.add(az2,qtTotal));
            jkxlFee.setAzFee(az2);
            jkxlFee.setQtFee(qtTotal);
            if(integrityFee!=0.0){
                cxkkFee.setTotalFee(integrityFee);
                midQtFee.setTotalFee(DoubleUtil.add(midQtFee.getTotalFee(),integrityFee));
                qtFee.setTotalFee(qtTotal);
            }
        }
        //        根据节点编码特征判断 本地节点、其他费用节点的费用类型为“建筑费”、“安装费”或者“其他费用”，并设置金额。除此之外上层节点需单独设置
        for(JkxlDetailFee fee : oriList){
            Double totalFee  = fee.getTotalFee();
            //建筑
            if(fee.getWbsCode().substring(0,2).equals("21") || fee.getWbsCode().substring(0,3).equals("491")){
                fee.setJzFee(totalFee);
            }
            //安装
            if(fee.getWbsCode().substring(0,2).equals("23") || fee.getWbsCode().substring(0,3).equals("493") || fee.getWbsCode().substring(0,2).equals("39")){
                fee.setAzFee(totalFee);
            }
            //其他费用
            if(totalFee != null && fee.getFeeType().equals("0")){
                fee.setQtFee(totalFee);
            }
        }
        return oriList;
    }

}
