package com.jeeplus.modules.sg.overheadline.util;

import com.jeeplus.modules.sg.overheadline.entity.JkxlAdjustFee;
import com.jeeplus.modules.sg.overheadline.entity.JkxlDetailFee;
import com.jeeplus.modules.sg.overheadline.entity.Settlement;
import com.jeeplus.modules.sg.overheadline.mapper.OverheadLineMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 项目展示工具类
 */


@Service
@Transactional(readOnly = true)
public class ItemShowUtil {

    @Autowired
    private OverheadLineMapper overheadLineMapper;

    /**
     * 为本体工程添加上级合计行，用于展示。同时设置最上级的parentNode为-1
     * 为了方便动态调整结算书数据以及展示与导出，我们只在数据库中保存本体工程类型节点（不包括“本体工程”节点）与其他费用类型节点
     * @param list
     * @param type
     */
    public JkxlDetailFee addTotalColumn(List<JkxlDetailFee> list,String type){
        if(type.equals("1")){
            JkxlDetailFee fee = overheadLineMapper.getElement("39000000");
            fee.setParentNode("-1");
            DetailFeeUtil.getGatherFee(fee,list);
            list.add(fee);
            return fee;
        }else if(type.equals("2")){
            JkxlDetailFee body = overheadLineMapper.getElement("49000000");
            body.setParentNode("-1");
            JkxlDetailFee jz = overheadLineMapper.getElement("49100000");
            JkxlDetailFee az = overheadLineMapper.getElement("49300000");
            List<JkxlDetailFee> jzList = new ArrayList<>();
            List<JkxlDetailFee> azList = new ArrayList<>();
            for(JkxlDetailFee fee:list){
                String wbsCode = fee.getWbsCode();
                if(wbsCode.startsWith("491")){
                    jzList.add(fee);
                }else if(wbsCode.startsWith("493")){
                    azList.add(fee);
                }
            }
            DetailFeeUtil.getGatherFee(jz,jzList);
            DetailFeeUtil.getGatherFee(az,azList);
            DetailFeeUtil.getGatherFee(body,list);
            list.add(body);
            list.add(jz);
            list.add(az);
            return body;
        }else if(type.equals("3")){
            JkxlDetailFee top = overheadLineMapper.getElement("20000000");
            top.setParentNode("-1");
            List<JkxlDetailFee> topList = new ArrayList<>();
            for(JkxlDetailFee fee:list){
                String wbsCode = fee.getWbsCode();
                if(wbsCode.equals("21000000")||wbsCode.equals("23000000")){
                    topList.add(fee);
                }
            }
            DetailFeeUtil.getGatherFee(top,topList);
            list.add(top);
            return top;
        }
        return null;
    }

    public List<Settlement> disposeData(JkxlDetailFee topBodyFee,JkxlDetailFee topOtherFee, List<Settlement> settlementList, List<JkxlAdjustFee> tzList) {
        double totalCe = 0.00;
        //总计算差额
        for(JkxlAdjustFee jkxlAdjustFee : tzList){
            if(jkxlAdjustFee.getType().equals(BashInfo.CE)){
                totalCe = jkxlAdjustFee.getFee();
            }
        }
        for(Settlement settlement : settlementList){
            //分部分项工程费差额
            if(settlement.getEntryName().contains(BashInfo.JS_FBFX)){
                settlement.setTotalFee(topBodyFee.getOriginalTotalCost());
                settlement.setDifference(DoubleUtil.sub(settlement.getMoney(),topBodyFee.getOriginalTotalCost()));
            }
            //发包人材料采购费
            if(settlement.getEntryName().contains("发包人") || settlement.getEntryName().contains("招标人")){
                settlement.setTotalFee(topBodyFee.getOriginalFbrCost());
                settlement.setDifference(DoubleUtil.sub(settlement.getMoney(),topBodyFee.getOriginalFbrCost()));
            }
            //措施费差额
            if(settlement.getEntryName().contains(BashInfo.JS_CSXM)){
                double measuresFee = DoubleUtil.add(topBodyFee.getMeasuresFee1(),topBodyFee.getMeasuresFee2(),topBodyFee.getMeasuresFee3());
                settlement.setTotalFee(measuresFee);
                settlement.setDifference(DoubleUtil.sub(settlement.getMoney(),measuresFee));
            }
            //规费差额
            if(settlement.getEntryName().contains(BashInfo.JS_GF)){
                settlement.setTotalFee(topBodyFee.getFees());
                settlement.setDifference(DoubleUtil.sub(settlement.getMoney(),topBodyFee.getFees()));
            }
            //税金差额
            if(settlement.getEntryName().contains(BashInfo.JS_SJ)){
                settlement.setTotalFee(topBodyFee.getTex());
                settlement.setDifference(DoubleUtil.sub(settlement.getMoney(),topBodyFee.getTex()));
            }
            //其他费用差额
            if(settlement.getEntryName().contains(BashInfo.JS_QTXM)){
                double otherFee = 0;
//                避免其他费用为空时报错
                if(topOtherFee!=null){
                    otherFee = topOtherFee.getTotalFee();
                }
                settlement.setTotalFee(DoubleUtil.add(topBodyFee.getOtherCost(),otherFee));
                settlement.setDifference(DoubleUtil.add(settlement.getMoney(),-topBodyFee.getOtherCost(),-otherFee));
            }
            //结算差额
            if(settlement.getEntryName().contains("结算") || settlement.getEntryName().contains("报价") || settlement.getEntryName().contains("下浮后") || settlement.getEntryName().contains("最高")){
                settlement.setTotalFee(topBodyFee.getTotalFee());
                settlement.setDifference(totalCe);
            }


        }
        return settlementList;
    }


}
