package com.jeeplus.modules.sg.project.service;


import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.sg.overheadline.entity.JkxlDetailFee;
import com.jeeplus.modules.sg.overheadline.service.OverheadLineService;
import com.jeeplus.modules.sg.project.entity.WbsItem;
import com.jeeplus.modules.sg.project.mapper.ItemMapper;
import com.jeeplus.modules.sg.project.util.ProjectUtil;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 用于处理以人工统计方式导入的结算书
 */

@Service
@Transactional(readOnly = true)
public class ManualItemService {

    private static final Map<String,String> WBS_MAP = new HashMap<>();
    static{
        WBS_MAP.put("防火封堵","23161400");
        WBS_MAP.put("火灾报警","21144000");
        WBS_MAP.put("三通（水、电、气、通讯）","21360000");
        WBS_MAP.put("一平（场地平整）","21221000");
        WBS_MAP.put("电缆桥支架安装","23161300");
    }

    @Autowired
    private ItemService itemService;
    @Autowired
    private OverheadLineService overheadLineService;

    public void save(MultipartFile file,String projectId) throws IOException, InvalidFormatException {
        ImportExcel excel = new ImportExcel(file,1,0);
        int lastRow = excel.getLastDataRowNum();
        Map<String,WbsItem> map = new HashMap<>();
        for(int i=1;i<lastRow;i++){
            String itemName = excel.getValue(i,0);
            double mount = excel.getDouble(i,1);
            double agencyFee = excel.getDouble(i,2);
            String unit = excel.getValue(i,3);
            String wbsCode = WBS_MAP.get(itemName);
            if(mount==0.0){
                continue;
            }
            if(wbsCode==null){
                throw new RuntimeException(file.getOriginalFilename()+"手动模板填写有误，工程施工类型：'"+itemName+"'不合规！");
            }
            if(StringUtils.isBlank(unit)){
                throw new RuntimeException("手动模板填写有误,工程施工类型：'"+itemName+"'行未填写施工单位！");
            }
            WbsItem item = new WbsItem();
            item.setItemName(itemName);
            item.setProjectId(projectId);
            List<WbsItem> list = itemService.findList(item);
            if(list!=null&&list.size()>0){
                throw new RuntimeException("手动模板填写有误，项目中已经存在结算书："+itemName);
            }
            item.setDelFlag("0");
            item.setType(getItemType(wbsCode));
            String id = ProjectUtil.getId();
            item.setId(id);
            item.setUnit(unit);
            itemService.save(item);
            overheadLineService.saveLayerFee(id,wbsCode,"1",mount);
//            通过观察得到招标代理费wbs规律
            String agencyWbs = wbsCode.substring(0,1)+"4220000";
            overheadLineService.saveLayerFee(id,agencyWbs,"0",agencyFee);
        }
    }

    private String getItemType(String wbsCode){
        if(wbsCode.startsWith("2")){
            return "3";
        }else if(wbsCode.startsWith("3")){
            return "1";
        }else if(wbsCode.startsWith("4")){
            return "2";
        }else{
            return null;
        }
    }
}
