package com.jeeplus.modules.sg.project.service;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.utils.sg.ExcelUtil;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.sg.overheadline.entity.JkxlDetailFee;
import com.jeeplus.modules.sg.overheadline.entity.Settlement;
import com.jeeplus.modules.sg.overheadline.mapper.OverheadLineMapper;
import com.jeeplus.modules.sg.overheadline.service.FeeAdjustService;
import com.jeeplus.modules.sg.overheadline.service.OverheadLineService;
import com.jeeplus.modules.sg.overheadline.service.SettlementService;
import com.jeeplus.modules.sg.overheadline.util.BashInfo;
import com.jeeplus.modules.sg.project.entity.WbsItem;
import com.jeeplus.modules.sg.project.entity.WbsProject;
import com.jeeplus.modules.sg.project.entity.WbsSelection;
import com.jeeplus.modules.sg.project.mapper.ItemMapper;
import com.jeeplus.modules.sg.project.mapper.ProjectMapper;
import com.jeeplus.modules.sg.project.util.ProjectUtil;
import com.jeeplus.modules.sg.substation.service.SubstationService;
import com.jeeplus.modules.sg.substation.util.SubstationInfo;
import com.jeeplus.modules.sys.entity.User;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service
@Transactional(readOnly = true)
public class ProjectService extends CrudService<ProjectMapper, WbsProject> {
    @Autowired
    private ProjectMapper projectMapper;
    @Autowired
    private OverheadLineService overheadLineService;
    @Autowired
    private SubstationService substationService;
    @Autowired
    private ManualItemService manualItemService;
    @Autowired
    private WbsItemService wbsItemService;
    @Autowired
    private ItemMapper itemMapper;

    /**
     * 获取单个工程 及工程下项目信息
     * @param id
     * @return
     */
    @Transactional(readOnly = false)
    public WbsProject get(String id) {
        WbsProject project =  projectMapper.get(id);
        WbsItem item = new WbsItem();
        item.setProjectId(id);
        List<WbsItem> items = itemMapper.findList(item);
        project.setWbsItems(items);
        return project;
    }

    /**
     * 保存工程与多个结算书及明细，返回工程id
     * @param file        结算书文件
     * @param projectName  工程名称
     * @param projectNo     工程编号
     * @param itemNames       项目（结算书）名称
     * @param units             供应商
     */
    @Transactional(readOnly = false)
    public String saveProjectDetail(MultipartFile[] file,String projectName,String projectNo,String[] itemNames,String[] units) throws Exception {
//        添加工程
        WbsProject wbsProject = new WbsProject();
        wbsProject.setProjectName(projectName);
        wbsProject.setProjectNo(projectNo);
        save(wbsProject);
        String projectId = wbsProject.getId();
        try{
            for(int i = 0;i<itemNames.length;i++) {
                WbsItem wbsItem = new WbsItem();
                wbsItem.setItemName(itemNames[i]);
                wbsItem.setProjectId(projectId);
                if (null != units && units.length > 0) {
                    if (StringUtils.isNotBlank(units[i])) {
                        wbsItem.setUnit(units[i]);
                    }
                }
                //获取唯一ID
                String id = ProjectUtil.getId();
                wbsItem.setId(id);
                //分析保存
                saveItemDetail(file[i], wbsItem);
            }
        }catch (Exception e){
            throw e;
        }
        return projectId;
    }


    /**
     * 保存结算书及明细
     * @param file
     * @param wbsItem
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     */
    @Transactional(readOnly = false)
    public String saveItemDetail(MultipartFile file ,WbsItem wbsItem) throws IOException, InvalidFormatException {
        //架空类型判断
        ImportExcel jk = new ImportExcel(file,1,new String[]{BashInfo.SHEET_GCHZ,BashInfo.SHEET_GCHZ1},false); //架空汇总
        if(ImportExcel.getSheet(jk) != null){
            wbsItem.setType(BashInfo.PROJECT_JKXL);
            overheadLineService.saveJk(file,wbsItem);
            return BashInfo.PROJECT_JKXL;
        }
        //电缆类型判断
        ImportExcel dljz = new ImportExcel(file, 1, BashInfo.SHEET_JZHZB,false);   //电缆线路建筑工程费用汇总表
        ImportExcel dlaz = new ImportExcel(file, 1, BashInfo.SHEET_HZHZB,false);   //电缆线路安装工程费用汇总表
        if(ImportExcel.getSheet(dlaz) != null || ImportExcel.getSheet(dljz) != null){
            wbsItem.setType(BashInfo.PROJECT_DLXL);
            overheadLineService.saveDl(file,wbsItem);
            return BashInfo.PROJECT_DLXL;
        }
        //变电站类型判断
        ImportExcel bdjz = new ImportExcel(file, 1, SubstationInfo.SHEET_JZHZB,false);   //变电站建筑工程费用汇总表
        ImportExcel bdaz = new ImportExcel(file, 1, SubstationInfo.SHEET_HZHZB,false);   //变电站安装工程费用汇总表
        if(ImportExcel.getSheet(bdjz) != null || ImportExcel.getSheet(bdaz) != null){
            wbsItem.setType(BashInfo.PROJECT_BDZ);
            substationService.save(file,wbsItem);
            return BashInfo.PROJECT_BDZ;
        }

        ImportExcel manual = new ImportExcel(file,1,"manual",false);//手动调整结算书
        if(ImportExcel.getSheet(manual)!=null){
            manualItemService.save(file,wbsItem.getProjectId());
            return BashInfo.PROJECT_MUL;
        }
        return wbsItemService.save(file,wbsItem);
    }


    /**
     * 分页查询工程及工程下项目
     * @param page 分页对象
     * @param selection 查询条件
     * @return
     */
    public Page<WbsProject> findPage(Page<WbsProject> page, WbsSelection selection) {
        WbsProject project = selection.getProject();
        WbsItem item = selection.getItem();
        //数据权限过滤
        dataRuleFilter(project);
        project.setPage(page);
        List<WbsProject> list =  projectMapper.findList(project);
        long count = page.getCount();
        Integer pageNo = page.getPageNo();
        //根据工程ID查询项目信息
        for(WbsProject pro : list){
            item.setProjectId(pro.getId());
            List<WbsItem> itemList = itemMapper.findList(item);
            pro.setWbsItems(itemList);
        }
        if(selection.hasItemCondition()){
            List<WbsProject> valuedList = new ArrayList<>();
            for(WbsProject pro : list){
                if(pro.getWbsItems().size()>0){
                    valuedList.add(pro);
                }
            }
            page.setCount(valuedList.size());
            page.setList(valuedList);
            return page;
        }
        //这里的处理是不展示没有符合条件项目的工程 做判断是防止某些工程本来就没有项目，而在没有这些过滤条件情况下应该展示工程
        page.setPageNo(pageNo);
        page.setCount(count);
        page.setList(list);
        return page;
    }


    /**
     * 更新工程信息，及工程下项目的供应商信息
     * @param wbsProject
     * @param wbsItems
     */
    @Transactional(readOnly = false)
    public void update(WbsProject wbsProject,List<WbsItem> wbsItems){
        projectMapper.update(wbsProject);
        itemMapper.updateList(wbsItems);
    }


    /**
     * 逻辑删除工程及工程下的项目
     * @param wbsProject
     * @return
     */
    @Transactional(readOnly = false)
    public void deleteProject(WbsProject wbsProject){
        itemMapper.deleteItems(wbsProject.getId());
        projectMapper.deleteProject(wbsProject);
    }


}
