package com.jeeplus.modules.sg.project.web;

import com.alibaba.fastjson.JSONArray;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.sg.overheadline.entity.JkxlDetailFee;
import com.jeeplus.modules.sg.overheadline.entity.Settlement;
import com.jeeplus.modules.sg.overheadline.service.OverheadLineService;
import com.jeeplus.modules.sg.overheadline.util.BashInfo;
import com.jeeplus.modules.sg.overheadline.util.DetailFeeUtil;
import com.jeeplus.modules.sg.project.entity.WbsItem;
import com.jeeplus.modules.sg.project.entity.WbsProject;
import com.jeeplus.modules.sg.project.entity.WbsSelection;
import com.jeeplus.modules.sg.project.service.ItemService;
import com.jeeplus.modules.sg.project.service.ProjectService;
import com.jeeplus.modules.sg.project.util.ExportUtil;
import com.jeeplus.modules.sg.project.util.ProjectUtil;
import com.jeeplus.modules.sg.substation.service.SubstationService;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.ui.Model;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;


@Controller
@RequestMapping("${adminPath}/project")
public class ProjectController extends BaseController {
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private ProjectService projectService;
    @Autowired
    private OverheadLineService overheadLineService;
    @Autowired
    private ItemService itemService;


    /**
     * 历史工程管理页面 工程项目数据
     */
    @RequestMapping(value = {"list", ""})
    public String list(WbsSelection selection, HttpServletRequest request, HttpServletResponse response, Model model) {
        Page<WbsProject> page = projectService.findPage(new Page<WbsProject>(request, response), selection);
        model.addAttribute("page",page);
        return "modules/sg/project/management";
    }

    /**
     *历史工程管理页面，单个工程信息查看编辑的跳转方法
     */
    @RequestMapping("/form")
    public String form(WbsProject wbsProject, Model model) {
        WbsProject project = projectService.get(wbsProject.getId());
        List<WbsProject> list = new ArrayList<>();
        list.add(project);
        model.addAttribute("list", list);
        String view = "updateItemForm";
        String tabId = request.getParameter("tabId");
        if("1".equals(tabId)){
            view = "lookItemForm";
        }
        return "modules/sg/project/"+view;
    }

    /**
     * 历史工程管理页面的新建工程（没有项目信息）
     * @param wbsProject
     * @param model
     * @param redirectAttributes
     * @return
     */
    @ResponseBody
    @RequestMapping("/save")
    public AjaxJson save(WbsProject wbsProject, Model model, RedirectAttributes redirectAttributes){
        AjaxJson j = new AjaxJson();
        //数据验证
        String errMsg = beanValidator(wbsProject);
        if (StringUtils.isNotBlank(errMsg)){
            j.setSuccess(false);
            j.setMsg(errMsg);
            return j;
        }
        //新增或编辑表单保存
        projectService.save(wbsProject);//保存
        j.setSuccess(true);
        addMessage(redirectAttributes, "保存成功");
        return j;
    }

    /**
     * 工程及下属项目信息修改
     * @param wbsProject  工程信息
     * @param ids  项目ID
     * @param units 项目供应商
     * @return
     */
    @ResponseBody
    @RequestMapping("/update")
    public Map update(WbsProject wbsProject,String ids,String units){
        Map map = new HashMap();
        WbsItem wbsItem = null;
        List<WbsItem> list = new ArrayList<>();
        try {
            String[] id = ids.split(",");
            String[] unit = units.split(",");
            for (int i = 0; i < id.length; i++) {
                wbsItem = new WbsItem();
                wbsItem.setId(id[i]);
                wbsItem.setUnit(unit[i]);
                list.add(wbsItem);
            }
            projectService.update(wbsProject, list);
        }catch (Exception e){
            map.put("code",0);
            map.put("msg","修改失败");
            return map;
        }
        map.put("code",1);
        map.put("msg","修改成功");
        return map;
    }

    /**
     *添加单个结算书
     */
    @RequestMapping("/import")
    public String importFile(MultipartFile file,WbsItem wbsItem, RedirectAttributes redirectAttributes, HttpServletRequest request,Model model) throws IOException, InvalidFormatException {
        String id = ProjectUtil.getId();
        wbsItem.setId(id);
        try {
           //判断结算数类型并保存数据到数据库
             String str = projectService.saveItemDetail(file,wbsItem);
             addMessage(redirectAttributes, "导入成功");
            if(str.equals(BashInfo.PROJECT_MUL)){
                return "redirect:/a/project/list";
            }
            return "redirect:/a/jkxl/list?id="+id+"&type="+str;
        }   catch (Exception e){
            addMessage(redirectAttributes, "导入失败,"+ProjectUtil.dealMsg(e.getMessage()));
            return "redirect:/a/project/list";
        }
    }

    /**
     * 导出整个工程的成果文件
     * @param request
     * @param model
     * @param response
     * @return
     * @throws IOException
     */
    @RequestMapping("/exportone")
    public String exportone(HttpServletRequest request,Model model,HttpServletResponse response) throws IOException {
//        获取工程项目信息
        String id = request.getParameter("id");
        WbsProject wbsProject = projectService.get(id);
        List<WbsItem> wbsItems = wbsProject.getWbsItems();
        List<String> unitList = new ArrayList<>();
        unitList.add("合计");
        Map<String,List<JkxlDetailFee>> map = new HashMap<>();
        for(WbsItem item:wbsItems){
            String unit = item.getUnit();
            if(!unitList.contains(unit)){
                unitList.add(unit);
            }
            List<JkxlDetailFee> mapFees = map.get(unit);
            List<JkxlDetailFee> fees = overheadLineService.findList(item.getId(),item.getType());
            if(mapFees==null){
                map.put(unit,fees);
            }else{
                mapFees.addAll(fees);
            }
        }
        String[] unitArr = unitList.toArray(new String[unitList.size()]);
        ExportUtil exportUtil = new ExportUtil(null, JkxlDetailFee.class,unitArr);
        for(int i=1;i<unitList.size();i++){
            List<JkxlDetailFee> combinedList = DetailFeeUtil.combineFees(map.get(unitList.get(i)));
            map.put(unitList.get(i),combinedList);
            exportUtil.setDataList(combinedList,i);
        }
        List<JkxlDetailFee> allFees = new ArrayList<>();
        for(List<JkxlDetailFee> fees:map.values()){
            allFees.addAll(fees);
        }
        List<JkxlDetailFee> comAllFees = DetailFeeUtil.combineFees(allFees);
        exportUtil.setDataList(comAllFees,0);

        //写入EXCAL
        String fileName = wbsProject.getProjectNo()+wbsProject.getProjectName()+".xlsx";
        exportUtil.write(response,fileName).dispose();
        return null;
    }


    /**
     * 财务处理调整页面
     */
    @RequestMapping("withholdform")
    public String withholdForm(String id,Model model){
        WbsProject wbsProject = projectService.get(id);
        if(wbsProject != null){
            List<WbsItem> list = wbsProject.getWbsItems();
            model.addAttribute("list",list);
        }
        return "modules/sg/project/withholdForm";
    }


    /**
     * 添加诚信扣款、甲供材超供抵扣工程款、税金处理
     * @param ids
     * @param fees          诚信扣款
     * @param unitTypes
     * @param decFunds      抵扣工程款
     * @param texs          税金处理
     * @return
     */
    @ResponseBody
    @RequestMapping("withhold")
    public AjaxJson withhold(String ids,String fees,String unitTypes,String decFunds,String texs){
        AjaxJson j = new AjaxJson();
        List<WbsItem> ts = new ArrayList<>();
        WbsItem wbsItem = null;
        try {
            String[] idArr = ids.split(",");
            String[] feeArr = fees.split(",");
            String[] unitTypeArr = unitTypes.split(",");
            String[] decFundArr = decFunds.split(",");
            String[] texArr = texs.split(",");
            for(int i=0;i<idArr.length;i++){
                wbsItem = new WbsItem();
                wbsItem.setId(idArr[i]);
//                单位工程 暂无用
                String unitType = unitTypeArr[i];
                if(unitType.equals("null")){
                    unitType = null;
                }
                wbsItem.setUnitType(unitType);
                String fee = feeArr[i];
                if(fee.equals("null")){
                    wbsItem.setIntegrityFee(null);
                }else {
                    wbsItem.setIntegrityFee(Double.valueOf(fee));
                }
                String decFund = decFundArr[i];
                if(decFund.equals("null")){
                    wbsItem.setDecFund(null);
                }else{
                    wbsItem.setDecFund(Double.valueOf(decFund));
                }
                String tex = texArr[i];
                if(tex.equals("null")){
                    wbsItem.setTexDeal(null);
                }else{
                    wbsItem.setTexDeal(Double.valueOf(tex));
                }
                ts.add(wbsItem);

            }
            itemService.updateFee(ts);
            j.setSuccess(true);
            j.setMsg("操作成功");
        }catch (Exception e){
            j.setSuccess(false);
            j.setMsg("操作失败");
        }
        return j;
    }

    /**
     * 逻辑删除工程及工程下项目
     */
    @RequestMapping("/deleteProject")
    @ResponseBody
    public AjaxJson deleteProject(WbsProject wbsProject){
        AjaxJson j = new AjaxJson();
        j.setSuccess(true);
        j.setMsg("删除工程成功!");
        try {
            projectService.deleteProject(wbsProject);
        }catch (Exception e){
            j.setSuccess(false);
            j.setMsg("删除工程失败!");
        }
        return j;
    }
}
