package com.jeeplus.modules.sg.project.web;

import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.sg.overheadline.service.OverheadLineService;
import com.jeeplus.modules.sg.overheadline.util.BashInfo;
import com.jeeplus.modules.sg.project.entity.WbsItem;
import com.jeeplus.modules.sg.project.entity.WbsProject;
import com.jeeplus.modules.sg.project.entity.WbsSelection;
import com.jeeplus.modules.sg.project.service.ItemService;
import com.jeeplus.modules.sg.project.service.ProjectService;
import com.jeeplus.modules.sg.project.util.ProjectUtil;
import com.jeeplus.modules.sg.substation.service.SubstationService;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 *单项工程管理
 */

@Controller
@RequestMapping("${adminPath}/project/tem")
public class ProjectTemController extends BaseController {
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private ProjectService projectService;
    @Autowired
    private ItemService itemService;


    /**
     *首页跳转  单项工程导入页面
     */
    @RequestMapping("/index")
    public String toIndex(){
        return "modules/sg/projecttem/addItem2Form";
    }

    /**
     * 单项工程管理页面 多个结算书导入
     * @param file 文件数组
     * @param projectName 单体工程名称
     * @param projectNo  单体工程编号
     * @param itemNames 结算书名称
     * @param units  供应商
     * @return
     */
    @ResponseBody
    @RequestMapping("/import")
    public HashMap importFile(@RequestParam(value = "file")MultipartFile[] file,String projectName,String projectNo,String[] itemNames,String[] units) {
        HashMap map = new HashMap();
        String projectId = "";
        try {
            projectId = projectService.saveProjectDetail(file,projectName,projectNo,itemNames,units);
        }  catch (Exception e){
            map.put("code",0);
            map.put("msg","导入失败，"+ProjectUtil.dealMsg(e.getMessage()));
            return map;
        }
        map.put("code",1);
        map.put("msg","导入成功");
        map.put("id",projectId);
        return map;
    }


    /**
     * 在单项工程导入页面导入结算书成功后跳转的 单个工程管理页面
     * @param wbsProject
     * @param request
     * @param response
     * @param model
     * @return
     */
    @RequestMapping(value = {"list", ""})
    public String list(WbsProject wbsProject, HttpServletRequest request, HttpServletResponse response, Model model) {
       WbsProject project = projectService.get(wbsProject.getId());
       //迁就前台页面放入一个list
       List<WbsProject> list = new ArrayList<>();
       list.add(project);
       model.addAttribute("list",list);
       return "modules/sg/projecttem/manageList";
    }


    /**
     * 单个工程管理页面 导入新结算书
     * @param file  文件
     * @param wbsItem 项目信息
     * @param redirectAttributes
     * @param request
     * @param model
     * @return
     */
    @RequestMapping("/importfile")
    public String importFile(MultipartFile file,WbsItem wbsItem, RedirectAttributes redirectAttributes, HttpServletRequest request, Model model) {
        String projectId = wbsItem.getProjectId();
        String id = ProjectUtil.getId();
        wbsItem.setId(id);
        try {
//            保存数据到数据库
            String type = projectService.saveItemDetail(file,wbsItem);
            addMessage(redirectAttributes, "导入成功");
            if(type.equals(BashInfo.PROJECT_MUL)){
                return "redirect:/a/project/tem/list?id="+projectId;
            }
            return "redirect:/a/jkxl/list?id="+id+"&type="+type+"&projectId="+projectId;
        }   catch (Exception e){
            addMessage(redirectAttributes, "导入失败，"+ProjectUtil.dealMsg(e.getMessage()));
            return "redirect:/a/project/tem/list?id="+projectId;
        }
    }


    /**
     * 单个工程管理页面  查看或修改工程信息
     * @param wbsProject  工程id与打开方式
     * @param model
     * @return
     */
    @RequestMapping("/form")
    public String form(WbsProject wbsProject, Model model) {
        WbsProject pro = projectService.get(wbsProject.getId());
        List<WbsProject> list = new ArrayList<>();
        list.add(pro);
        model.addAttribute("list", list);
//        如果tabId为1表示查看 否认表示修改
        String view = "updateItem";
        String tabId = request.getParameter("tabId");
        if("1".equals(tabId)){
            view = "lookItem";
        }
        return "modules/sg/projecttem/"+view;
    }

    /**
     * 检验工程下是否存在某项目名称
     * @param itemName      项目名称
     * @param projectId
     * @return
     */
    @RequestMapping("/verfiy")
    @ResponseBody
    public HashMap getVerfiy(String itemName,String projectId){
        HashMap map = new HashMap();
        WbsItem item = new WbsItem();
        item.setProjectId(projectId);
        item.setItemName(itemName);
        List<WbsItem> itemList = itemService.findList(item);
        if (itemList!=null&&itemList.size()>0){
            map.put("msg","导入失败! 结算书名称:"+itemName+" 已存在");
            return map;
        }
        map.put("code",1);
        map.put("msg","验证通过");
        return map;
    }

    /**
     * 属于历史工程管理页面中 删除单个项目
     * @param wbsItem
     * @return
     */
    @RequestMapping("/deleteItem")
    @ResponseBody
    public AjaxJson deleteItem(WbsItem wbsItem){
        AjaxJson j = new AjaxJson();
        try {
            WbsItem item = itemService.get(wbsItem.getId());
            itemService.deleteItemOne(item);
            j.setSuccess(true);
            j.setMsg(item.getProjectId());
        }catch (Exception e){
            j.setSuccess(false);
            j.setMsg("删除结算书失败!");
        }
        return j;
    }


}
